@@ -9,8 +9,15 @@ import logging
 import base64
 import threading
 from typing import (
-    Any, Dict, List, Type, Callable,
-    Optional, DefaultDict, Union, Tuple,
+    Any,
+    Dict,
+    List,
+    Type,
+    Callable,
+    Optional,
+    DefaultDict,
+    Union,
+    Tuple,
 )
 from types import FrameType
 from collections import defaultdict
@@ -34,6 +41,7 @@ class AuthorizationState(enum.Enum):
     WAIT_TDLIB_PARAMETERS = 'authorizationStateWaitTdlibParameters'
     WAIT_ENCRYPTION_KEY = 'authorizationStateWaitEncryptionKey'
     WAIT_PHONE_NUMBER = 'authorizationStateWaitPhoneNumber'
+    WAIT_REGISTRATION = 'authorizationStateWaitRegistration'
     READY = 'authorizationStateReady'
     CLOSING = 'authorizationStateClosing'
     CLOSED = 'authorizationStateClosed'
@@ -116,9 +124,7 @@ class Telegram:
         self._stopped = threading.Event()
 
         # todo: move to worker
-        self._workers_queue: queue.Queue = queue.Queue(
-            maxsize=default_workers_queue_size
-        )
+        self._workers_queue: queue.Queue = queue.Queue(maxsize=default_workers_queue_size)
 
         if not worker:
             worker = SimpleWorker
@@ -222,9 +228,16 @@ class Telegram:
 
         return self.call_method('getUser', params={'user_id': user_id})
 
-    def get_chats(
-        self, offset_order: int = 0, offset_chat_id: int = 0, limit: int = 100
-    ) -> AsyncResult:
+    def get_user_full_info(self, user_id: int) -> AsyncResult:
+        """
+        Requests the full information about a user with id = user_id.
+
+        https://core.telegram.org/tdlib/docs/classtd_1_1td__api_1_1get_user_full_info.html
+        """
+
+        return self.call_method('getUserFullInfo', params={'user_id': user_id})
+
+    def get_chats(self, offset_order: int = 0, offset_chat_id: int = 0, limit: int = 100) -> AsyncResult:
         """
         Returns a list of chats:
 
@@ -275,7 +288,11 @@ class Telegram:
 
         return self._send_data(data)
 
-    def get_message(self, chat_id: int, message_id: int,) -> AsyncResult:
+    def get_message(
+        self,
+        chat_id: int,
+        message_id: int,
+    ) -> AsyncResult:
         """
         Return a message via its message_id
 
@@ -302,9 +319,7 @@ class Telegram:
         }
         return self._send_data(data)
 
-    def delete_messages(
-        self, chat_id: int, message_ids: List[int], revoke: bool = True
-    ) -> AsyncResult:
+    def delete_messages(self, chat_id: int, message_ids: List[int], revoke: bool = True) -> AsyncResult:
         """
         Delete a list of messages in a chat
 
@@ -329,9 +344,7 @@ class Telegram:
         Args:
             supergroup_id
         """
-        return self._send_data(
-            {'@type': 'getSupergroupFullInfo', 'supergroup_id': supergroup_id}
-        )
+        return self._send_data({'@type': 'getSupergroupFullInfo', 'supergroup_id': supergroup_id})
 
     def create_basic_group_chat(self, basic_group_id: int) -> AsyncResult:
         """
@@ -340,13 +353,9 @@ class Telegram:
         Args:
             basic_group_id
         """
-        return self._send_data(
-            {'@type': 'createBasicGroupChat', 'basic_group_id': basic_group_id}
-        )
+        return self._send_data({'@type': 'createBasicGroupChat', 'basic_group_id': basic_group_id})
 
-    def get_web_page_instant_view(
-        self, url: str, force_full: bool = False
-    ) -> AsyncResult:
+    def get_web_page_instant_view(self, url: str, force_full: bool = False) -> AsyncResult:
         """
         Use this method to request instant preview of a webpage.
         Returns error with 404 if there is no preview for this webpage.
@@ -396,14 +405,10 @@ class Telegram:
                 self._update_async_result(update)
                 self._run_handlers(update)
 
-    def _update_async_result(
-        self, update: Dict[Any, Any]
-    ) -> typing.Optional[AsyncResult]:
+    def _update_async_result(self, update: Dict[Any, Any]) -> typing.Optional[AsyncResult]:
         async_result = None
 
-        _special_types = (
-            'updateAuthorizationState',
-        )  # for authorizationProcess @extra.request_id doesn't work
+        _special_types = ('updateAuthorizationState',)  # for authorizationProcess @extra.request_id doesn't work
 
         if update.get('@type') in _special_types:
             request_id = update['@type']
@@ -416,9 +421,7 @@ class Telegram:
             async_result = self._results.get(request_id)
 
         if not async_result:
-            logger.debug(
-                'async_result has not been found in by request_id=%s', request_id
-            )
+            logger.debug('async_result has not been found in by request_id=%s', request_id)
         else:
             done = async_result.parse_update(update)
             if done:
@@ -477,7 +480,13 @@ class Telegram:
         return async_result
 
     def idle(
-        self, stop_signals: Tuple = (signal.SIGINT, signal.SIGTERM, signal.SIGABRT, signal.SIGQUIT)
+        self,
+        stop_signals: Tuple = (
+            signal.SIGINT,
+            signal.SIGTERM,
+            signal.SIGABRT,
+            signal.SIGQUIT,
+        ),
     ) -> None:
         """
         Blocks until one of the exit signals is received.
@@ -534,26 +543,28 @@ class Telegram:
          - AuthorizationState.WAIT_PASSWORD if a telegram password is required.
            The caller should ask the telegram password
            to the end user and then call send_password(password)
-         - AuthorizationState.READY if the login process scceeded.
+         - AuthorizationState.WAIT_REGISTRATION if a the user must finish registration
+           The caller should ask the first and last names
+           to the end user and then call register_user(first, last)
+         - AuthorizationState.READY if the login process succeeded.
         """
         if self.proxy_server:
             self._send_add_proxy()
 
-        actions: Dict[
-            AuthorizationState,
-            Callable[[], AsyncResult],
-        ] = {
+        actions: Dict[AuthorizationState, Callable[[], AsyncResult]] = {
             AuthorizationState.NONE: self.get_authorization_state,
             AuthorizationState.WAIT_TDLIB_PARAMETERS: self._set_initial_params,
             AuthorizationState.WAIT_ENCRYPTION_KEY: self._send_encryption_key,
             AuthorizationState.WAIT_PHONE_NUMBER: self._send_phone_number_or_bot_token,
             AuthorizationState.WAIT_CODE: self._send_telegram_code,
             AuthorizationState.WAIT_PASSWORD: self._send_password,
+            AuthorizationState.WAIT_REGISTRATION: self._register_user,
         }
 
         blocking_actions = (
             AuthorizationState.WAIT_CODE,
             AuthorizationState.WAIT_PASSWORD,
+            AuthorizationState.WAIT_REGISTRATION,
         )
 
         if self.phone:
@@ -562,9 +573,7 @@ class Telegram:
             logger.info('[login] Login process has been started with bot token')
 
         while self.authorization_state != AuthorizationState.READY:
-            logger.info(
-                '[login] current authorization state: %s', self.authorization_state
-            )
+            logger.info('[login] current authorization state: %s', self.authorization_state)
 
             if not blocking and self.authorization_state in blocking_actions:
                 return self.authorization_state
@@ -707,3 +716,38 @@ class Telegram:
         self.authorization_state = self._wait_authorization_result(result)
 
         return self.authorization_state
+
+    def _register_user(self, first: Optional[str] = None, last: Optional[str] = None) -> AsyncResult:
+        logger.info('Registering user')
+        if first is None:
+            first = input('Enter first name: ')
+        if last is None:
+            last = input('Enter last name: ')
+
+        data = {
+            '@type': 'registerUser',
+            'first_name': first,
+            'last_name': last,
+        }
+        return self._send_data(data, result_id='updateAuthorizationState')
+
+    def register_user(self, first: str, last: str) -> AuthorizationState:
+        """
+        Finishes the new user registration process
+
+        Args:
+          first the user's first name
+          last the user's last name
+          If either argument is None, it will be asked to the user using the input() function
+
+        Returns
+         - AuthorizationState. The called have to call `login` to continue the login process.
+
+        Raises:
+          - RuntimeError if the login failed
+
+        """
+        result = self._register_user(first, last)
+        self.authorization_state = self._wait_authorization_result(result)
+
+        return self.authorization_state
